"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CommandList_1 = __importDefault(require("../Models/CommandList"));
const Logs_1 = __importDefault(require("../Database/Models/Logs"));
const EMPTY_COMMAND = {
    AllowedChannels: [],
    AllowedRoles: [],
    AllowedUsers: [],
    Data: {}
};
class Processor {
    constructor(context, prefix = '!', logging = 0) {
        this.Context = context;
        this.Prefix = prefix;
        this.Loaded = false;
        this.Logging = logging;
    }
    async LoadCommandsLocal(Commands) {
        const data = new CommandList_1.default();
        Object.keys(Commands).forEach((key) => {
            const { AllowedChannels, AllowedRoles, AllowedUsers } = EMPTY_COMMAND;
            data[key.toLowerCase()] = new Commands[key](AllowedChannels, AllowedRoles, AllowedUsers, false);
            console.log('[SUCCESS] Loaded Command', key);
        });
        return data;
    }
    async LoadCommandsFromDatabase(Commands) {
        const data = new CommandList_1.default();
        const databaseCommands = this.Context.State.Commands;
        Object.keys(Commands).forEach((key) => {
            const commandData = databaseCommands[key] || Object.assign({}, EMPTY_COMMAND, { Namespace: key });
            const { AllowedChannels, AllowedRoles, AllowedUsers, Data } = commandData;
            data[key.toLowerCase()] = new Commands[key](AllowedChannels, AllowedRoles, AllowedUsers, true);
            data[key.toLowerCase()].Data = Data;
            console.log('[SUCCESS] Loaded Command', key);
        });
        return data;
    }
    formatCommandSignatures(signatures) {
        const maxLength = signatures.reduce((a, b) => b.Name.length > a.Name.length ? b : a).Name.length;
        const formattedSignatures = signatures.map(({ Name, Signature }) => `${Name.padEnd(maxLength, ' ')} | ${Signature}`);
        formattedSignatures.unshift('-'.padEnd(30));
        formattedSignatures.unshift(`${'Command'.padEnd(maxLength, ' ')} | Signature`);
        formattedSignatures.unshift('```');
        formattedSignatures.push('```');
        return formattedSignatures;
    }
    GetCommandSignatures(message) {
        return Object.values(this.Commands)
            .filter((command) => command.Validate(message))
            .map((command) => ({
            Name: command.Name(),
            Signature: command.Signature
        }));
    }
    HelpCommand(message) {
        const commands = this.GetCommandSignatures(message);
        if (commands.length > 0) {
            return message.channel.send(this.formatCommandSignatures(commands).join('\n'));
        }
        return message.channel.send('No Commands Found');
    }
    async LoadCommands(Commands, useDB) {
        if (this.Context.Loading === true) {
            const deferred = new Promise((resolve) => setTimeout(resolve, 300));
            await deferred;
        }
        if (useDB) {
            this.Commands = await this.LoadCommandsFromDatabase(Commands);
        }
        else {
            this.Commands = await this.LoadCommandsLocal(Commands);
        }
        this.DbInUse = useDB;
        this.Loaded = true;
        return this.Commands;
    }
    async SaveLogs(log) {
        if (this.DbInUse === false) {
            return false;
        }
        if (this.Logging >= 2 || (this.Logging === 1 && log.Command)) {
            await log.save();
            return true;
        }
        return false;
    }
    async Handle(message) {
        const logs = new Logs_1.default();
        const start = new Date().getTime();
        logs.User = message.author.username;
        logs.Channel = message.channel.id;
        try {
            let result;
            if (this.Context.Loading === true || this.Loaded === false) {
                console.log('[WARNING] Message Received before Data is loaded, delaying');
                setTimeout(() => this.Handle(message), Processor.DELAY_TIMER);
                return;
            }
            if (message.content.indexOf(this.Prefix) !== 0) {
                return Promise.resolve('');
            }
            const name = message.content.split(' ').shift().substr(1);
            const isValid = this.Commands[name.toLowerCase()];
            if (isValid) {
                logs.Command = name.toLowerCase();
                result = await this.Commands[name.toLowerCase()].Call(message);
            }
            else if (name === 'help') {
                result = await this.HelpCommand(message);
            }
            logs.Runtime = new Date().getTime() - start;
            logs.Success = true;
            await this.SaveLogs(logs);
            return result;
        }
        catch (e) {
            logs.Runtime = new Date().getTime() - start;
            logs.Success = false;
            logs.Response = e.message;
            await this.SaveLogs(logs);
            console.error('[Fatal Error]', e);
            return Promise.reject(e);
        }
    }
}
Processor.DELAY_TIMER = 300;
exports.default = Processor;
//# sourceMappingURL=Processor.js.map